==> agnostos-0.9.7.tar.gz: agnostos/agnostos_docroot/weekly_report.pl <==
#!/usr/bin/perl -w

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.
   
use taskman_lib;
use strict;
use CGI;
use Text::Wrap;

#--------------------------------------------------------------------------
# indent($string, $spacer);
# Returns $string formatted with $spacer at the beginning of each line
sub 
indent
{
    my ($string, $spacer) = @_;
    my ( $output, @lines, $line );
    @lines = split( /\r\n/, $string );
    foreach $line (@lines) {
	$output .= $spacer.$line."\n";
    }
    return $output;
}

#--------------------------------------------------------------------------
# underline ($string, $char)
# Returns $string underline with $char.
sub     
underline 
{
    my ($string, $char) = @_;
    my $output = "$string\n";
    for (my $i=0; $i<length($string); $i++) {
	$output .= $char;
    }
    return $output;
}
    

#--------------------------------------------------------------------------
# status_report($dbh,$table,$id,$status);
sub
status_report
{
    my ($dbh, $tableref, $status, $users_ref, $groups_ref) = @_;
    my %users = %$users_ref;
    my %groups = %$groups_ref;
    my ($title, $description, $note, $completed_date, $userid, $groupid, $depid);
    my ( $output, $dbq, $sql_query );
    my %stat_desc = ( 'in-progress', 'In Progress',
		      'queued'     , 'Scheduled for Next Week',
		      'wait'       , 'Other Tasks',
		      'done'       , 'Completed' );

    $output = underline( $stat_desc{$status}, '=' );
    $output .= "\n\n";

    $sql_query = "SELECT title, description, note, completed_date, 
                         assigned.userid, assigned.groupid, assigned.depid
                  FROM todo, assigned
                  WHERE status = '$status'
                  AND todo.todoid = assigned.todoid ";
    my $asstable;
    foreach $asstable (keys %$tableref) {
	my $assigned_table = gen_assigned_name($asstable);
	$sql_query .= "AND assigned.$assigned_table = $$tableref{$asstable} ";
    }
    if ($status eq 'done') {
	$sql_query .= "AND  (TO_DAYS(CURDATE()) - TO_DAYS(completed_date) < 8) ";
    }
    $sql_query .="ORDER BY assigned.groupid";
    $dbq = $dbh -> prepare($sql_query);
    $dbq -> execute;
    my $last_groupid=1; 
    while ( ($title, $description, $note, $completed_date, $userid, $groupid, $depid) 
	= $dbq -> fetchrow_array ) {
	if ($groupid != $last_groupid) {
	    $last_groupid = $groupid;
	    if ( $groupid == 1 ) {
		$output .= underline("Others",'-')."\n\n";
	    }
	    else {
		$output .= underline($groups{$groupid},'-')."\n\n";
	    }
	}
	$Text::Wrap::columns = 75;
	my $title_out = "* $title ($users{$userid})";
	# $title_out   .= " $completed_date" if ($status eq 'done');
	$title_out   .= "\n";
	$output .= wrap("","  ",$title_out);
	$Text::Wrap::columns = 70;
	my $indent = "     ";
	$description =~ s/\n$// ;
	$description = wrap($indent,$indent,$description);
	$output .= $description."\n\n";
    }
    $dbq -> finish;
    $output .= "\n---------------------------------------------------------------------------\n\n";
    return $output;
}

#---------------------------------------------------------------------------
# main

# Initialisation
my $query = new CGI;
my $dbh   = dbserver_connect('agnostos');
my ($title, $description, $note, $completed_date, $userid, $groupid, $depid);
my ($dbq, $ent_name);
 
my %users  = fetch_ents( 'user',       $dbh);
my %groups = fetch_ents( 'workgroup',  $dbh);
my %depts  = fetch_ents( 'department', $dbh);

$users{1} = "Unassigned";

# Get cookies
my $tableref = get_cookie ($query);

if (defined $$tableref{'fubar'} && $$tableref{'fubar'} eq 'completely' ) {
    print html_header("No view selected",$query);
    print "<h1>Error - no view selected</h1>".
	"You must select a user, group or department from the front page, ".
	"and click on 'View Entries' before a weekly report can be generated.<p>".
        "  Once you have done so, a cookie will".
	" be set, and you can then use the &quot;Weekly Report&quot; link.";
    print html_footer;
    $dbh -> disconnect;
    exit 0;
}

my $ent_table;
$ent_name = "";
foreach $ent_table (keys %$tableref) {
    my ($act_id, $act_name) = get_table($ent_table);
    $dbq = $dbh -> prepare("SELECT $act_name FROM $ent_table WHERE $act_id = $$tableref{$ent_table}");
    $dbq -> execute;
    if (length($ent_name) > 0) {
	$ent_name .= " and ";
    }
    $ent_name .= $dbq -> fetchrow_array;
    $dbq -> finish;
}    

print $query -> header('text/plain');
print underline("Weekly Report: $ent_name",'=')."\n\n";


my $status;
my @stati = ('done','in-progress','queued','wait');
foreach $status (@stati) {
    print status_report( $dbh, $tableref, $status, \%users, \%groups);
}
 


$dbh -> disconnect();

==> agnostos-0.9.10.tar.gz: agnostos/agnostos_docroot/weekly_report.pl <==
#!/usr/bin/perl -w

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.
   
use taskman_lib;
use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use Text::Wrap;
use Time::Local;

#--------------------------------------------------------------------------
# indent($string, $spacer);
# Returns $string formatted with $spacer at the beginning of each line
sub 
indent
{
    my ($string, $spacer) = @_;
    my ( $output, @lines, $line );
    @lines = split( /\r\n/, $string );
    foreach $line (@lines) {
	$output .= $spacer.$line."\n";
    }
    return $output;
}

#--------------------------------------------------------------------------
# underline ($string, $char)
# Returns $string underline with $char.
sub     
underline 
{
    my ($string, $char) = @_;
    my $output = "$string\n";
    for (my $i=0; $i<length($string); $i++) {
	$output .= $char;
    }
    return $output;
}
    

#--------------------------------------------------------------------------
# status_report($dbh,$table,$id,$status);
sub
status_report
{
    my ($dbh, $tableref, $status, $users_ref, $groups_ref, $end_time, $days_diff) = @_;
    my %users = %$users_ref;
    my %groups = %$groups_ref;
    my ($title, $description, $note, $completed_date, $userid, $groupid, $depid);
    my ( $output, $dbq, $sql_query );
    my %stat_desc = ( 'in-progress', 'In Progress',
		      'queued'     , 'Scheduled for Next Week',
		      'wait'       , 'Other Tasks',
		      'done'       , 'Completed' );

    $output = underline( $stat_desc{$status}, '=' );
    $output .= "\n\n";

    $sql_query = "SELECT title, description, note, completed_date, 
                         assigned.userid, assigned.groupid, assigned.depid
                  FROM todo, assigned
                  WHERE status = '$status'
                  AND todo.todoid = assigned.todoid ";
    my $asstable;
    foreach $asstable (keys %$tableref) {
	my $assigned_table = gen_assigned_name($asstable);
	$sql_query .= "AND assigned.$assigned_table = $$tableref{$asstable} ";
    }
    if ($status eq 'done') {
	$sql_query .= "AND  ((TO_DAYS(FROM_UNIXTIME($end_time))) - TO_DAYS(completed_date) < $days_diff) ";
    }
    $sql_query .="ORDER BY assigned.groupid";
    $dbq = $dbh -> prepare($sql_query);
    $dbq -> execute;
    my $last_groupid=1; 
    while ( ($title, $description, $note, $completed_date, $userid, $groupid, $depid) 
	= $dbq -> fetchrow_array ) {
	if ($groupid != $last_groupid) {
	    $last_groupid = $groupid;
	    if ( $groupid == 1 ) {
		$output .= underline("Others",'-')."\n\n";
	    }
	    else {
		$output .= underline($groups{$groupid},'-')."\n\n";
	    }
	}
	$Text::Wrap::columns = 75;
	my $title_out = "* $title ($users{$userid})";
	# $title_out   .= " $completed_date" if ($status eq 'done');
	$title_out   .= "\n";
	$output .= wrap("","  ",$title_out);
	$Text::Wrap::columns = 70;
	my $indent = "     ";
	$description =~ s/\n$// ;
	$description = wrap($indent,$indent,$description);
	$output .= $description."\n\n";
    }
    $dbq -> finish;
    $output .= "\n---------------------------------------------------------------------------\n\n";
    return $output;
}

#---------------------------------------------------------------------------
# main

# Initialisation
my $query = new CGI;
my %config = get_config();
my $dbh   = dbserver_connect(\%config);
my ($title, $description, $note, $completed_date, $userid, $groupid, $depid);
my ($dbq, $ent_name);
 
my %users  = fetch_ents( 'user',       $dbh);
my %groups = fetch_ents( 'workgroup',  $dbh);
my %depts  = fetch_ents( 'department', $dbh);

$users{1} = "Unassigned";

# Get cookies
my $tableref = get_cookie ($query);

if (defined $$tableref{'fubar'} && $$tableref{'fubar'} eq 'completely' ) {
    print html_header("No view selected",$query);
    print "<h1>Error - no view selected</h1>".
	"You must select a user, group or department from the front page, ".
	"and click on 'View Entries' before a weekly report can be generated.<p>".
        "  Once you have done so, a cookie will".
	" be set, and you can then use the &quot;Weekly Report&quot; link.";
    print html_footer;
    $dbh -> disconnect;
    exit 0;
}

# Set up start and end dates for the reports.

my $start_day   = $query -> param('start_day');
my $start_month = $query -> param('start_month');
my $start_year  = $query -> param('start_year');
my $end_day     = $query -> param('end_day');
my $end_month   = $query -> param('end_month');
my $end_year    = $query -> param('end_year');

my $start_time  = timelocal(0, 0, 0, $start_day, ($start_month-1), $start_year);
my $end_time    = timelocal(59, 59, 23, $end_day, ($end_month-1), $end_year);
my $days_diff   = int(($end_time-$start_time)/(3600*24));
my ($sep, @date_format) = parse_date_format(\%config);

my $ent_table;
$ent_name = "";
foreach $ent_table ('user','workgroup','department') {
    if (defined $$tableref{$ent_table}) {
	my ($act_id, $act_name) = get_table($ent_table);
	$dbq = $dbh -> prepare("SELECT $act_name FROM $ent_table WHERE $act_id = $$tableref{$ent_table}");
	$dbq -> execute;
	$ent_name .= ucfirst($ent_table).": ". $dbq -> fetchrow_array ."\n";
	$dbq -> finish;
    }
}    

print $query -> header('text/plain'); 
print underline("Report for the period: ".
		format_text_date($start_day,$start_month,$start_year,$sep,@date_format)." to ".
		format_text_date($end_day,$end_month,$end_year,$sep,@date_format)."\n$ent_name",'=')."\n\n";

my $status;
my @stati = ('done','in-progress','queued','wait');
foreach $status (@stati) {
    print status_report( $dbh, $tableref, $status, \%users, \%groups, $end_time, $days_diff);
}
 


$dbh -> disconnect();
